@@ -4,6 +4,7 @@ use deqp_runner::JunitGeneratorOptions;
 use deqp_runner::{
     parallel_deqp, parse_regex_set, read_lines, Deqp, DeqpCommand, RunnerResults, TestCase,
 };
+use log::*;
 use serde::Deserialize;
 use std::collections::HashMap;
 use std::fs::{self, File};
@@ -80,7 +81,7 @@ pub struct Run {
         long = "include-tests",
         help = "regexes of tests to include (non-matching tests are skipped)"
     )]
-    include: Option<Vec<String>>,
+    include: Vec<String>,
 
     #[structopt(
         long,
@@ -146,6 +147,20 @@ pub struct Run {
     )]
     env: Vec<(String, String)>,
 
+    #[structopt(
+        long,
+        default_value = "",
+        help = "regex to match against the GL_RENDERER or VK deviceName"
+    )]
+    renderer_check: String,
+
+    #[structopt(
+        long,
+        default_value = "",
+        help = "regex to match against the GL_VERSION"
+    )]
+    version_check: String,
+
     #[structopt(help = "arguments to deqp binary")]
     deqp_args: Vec<String>,
 }
@@ -185,6 +200,13 @@ pub struct Suite {
     )]
     flakes: Option<Vec<PathBuf>>,
 
+    #[structopt(
+        short = "t",
+        long = "include-tests",
+        help = "regexes of tests to include (non-matching tests are skipped)"
+    )]
+    include: Vec<String>,
+
     #[structopt(long, default_value = "1", help = "Runs 1 out of every N tests.")]
     fraction: usize,
 
@@ -243,7 +265,7 @@ struct DeqpConfig {
     baseline: Option<PathBuf>,
 
     #[serde(default)]
-    include: Option<Vec<String>>,
+    include: Vec<String>,
 
     #[serde(default)]
     deqp_args: Vec<String>,
@@ -268,6 +290,12 @@ struct DeqpConfig {
 
     #[serde(default)]
     prefix: String,
+
+    #[serde(default)]
+    renderer_check: String,
+
+    #[serde(default)]
+    version_check: String,
 }
 
 impl DeqpConfig {
@@ -306,7 +334,11 @@ impl DeqpConfig {
         })
     }
 
-    fn test_groups<'d, D: Deqp>(&self, deqp: &'d D) -> Result<Vec<(&'d D, Vec<TestCase>)>> {
+    fn test_groups<'d>(
+        &self,
+        deqp: &'d DeqpCommand,
+        filters: &[String],
+    ) -> Result<Vec<(&'d DeqpCommand, Vec<TestCase>)>> {
         if self.tests_per_group < 1 {
             eprintln!("tests_per_group must be >= 1.");
             std::process::exit(1);
@@ -314,23 +346,21 @@ impl DeqpConfig {
 
         let rayon_threads = rayon::current_num_threads();
 
-        let include_filter = if let Some(include) = &self.include {
-            Some(parse_regex_set(include).context("compiling include filters")?)
-        } else {
-            None
-        };
+        let mut include_filters = Vec::new();
+        if !self.include.is_empty() {
+            include_filters
+                .push(parse_regex_set(&self.include).context("compiling include filters")?);
+        }
+        if !filters.is_empty() {
+            include_filters.push(parse_regex_set(filters).context("compiling include filters")?);
+        }
+
         let test_names: Vec<TestCase> = read_lines(&self.caselists)?
             .into_iter()
             .map(TestCase::Deqp)
             .skip(self.fraction_start - 1)
             .step_by(self.fraction)
-            .filter(|test| {
-                if let Some(include_filter) = &include_filter {
-                    include_filter.is_match(test.name())
-                } else {
-                    true
-                }
-            })
+            .filter(|test| include_filters.iter().all(|x| x.is_match(test.name())))
             .collect::<Vec<TestCase>>();
 
         let tests_per_group = usize::max(
@@ -369,6 +399,70 @@ impl DeqpConfig {
             self.tests_per_group
         };
 
+        if !test_names.is_empty()
+            && (!self.renderer_check.is_empty() || !self.version_check.is_empty())
+        {
+            // Look at the testcases in the caselist to decide how to probe the
+            // renderer.  Note that we do some inference, because the caselist
+            // might not contain dEQP-GLESn.info.renderer or whatever in it.
+            //
+            // The alternative would be to use the testcase binary to infer, but
+            // then we don't know if we should be printing
+            // KHR-GLES32.info.renderer or KHR-GL33.info.renderer or whatever.
+            let deqp_version = test_names[0]
+                .name()
+                .split('.')
+                .next()
+                .context("finding dEQP case prefix")?;
+
+            let (renderer_testcase, version_testcase) = match deqp_version {
+                "dEQP-VK" => ("dEQP-VK.info.device".to_owned(), "".to_owned()),
+                "KHR-GLES2" | "KHR-GLES3" | "KHR-GLES31" | "dEQP-EGL" => {
+                    anyhow::bail!(
+                        "Can't do a renderer check based on testcase name {}",
+                        test_names[0].name()
+                    );
+                }
+                _ => (
+                    format!("{}.info.renderer", &deqp_version),
+                    format!("{}.info.version", &deqp_version),
+                ),
+            };
+
+            let default_regex = ".*";
+
+            let renderer_check = if self.renderer_check.is_empty() {
+                default_regex
+            } else {
+                &self.renderer_check
+            };
+            let version_check = if self.renderer_check.is_empty() {
+                default_regex
+            } else {
+                &self.version_check
+            };
+
+            // Make sure that we test (and thus log) both renderer and version
+            // before exiting out due to a mismatch.
+            let renderer_ok = deqp
+                .qpa_text_check(&renderer_testcase, renderer_check, "renderer")
+                .context("checking renderer")?;
+            let version_ok = version_testcase.is_empty()
+                || deqp
+                    .qpa_text_check(&version_testcase, version_check, "version")
+                    .context("checking version")?;
+
+            if !renderer_ok {
+                error!("Renderer mismatch ({})", &self.renderer_check);
+                std::process::exit(1);
+            }
+
+            if !version_ok {
+                error!("Version mismatch ({})", &self.version_check);
+                std::process::exit(1);
+            }
+        }
+
         println!(
             "Running dEQP on {} threads in {}-test groups",
             rayon_threads, tests_per_group
@@ -444,10 +538,12 @@ fn main() -> Result<()> {
                 fraction: run.fraction,
                 env: run.env.into_iter().collect(),
                 prefix: "".to_owned(),
+                renderer_check: run.renderer_check,
+                version_check: run.version_check,
             };
             let deqp = config.deqp(&run.output_dir, run.testlog_to_xml)?;
 
-            let results = parallel_deqp(std::io::stdout(), config.test_groups(&deqp)?)?;
+            let results = parallel_deqp(std::io::stdout(), config.test_groups(&deqp, &[])?)?;
             process_results(&results, &run.output_dir, run.summary_limit)?;
         }
 
@@ -524,7 +620,7 @@ fn main() -> Result<()> {
 
             let mut test_groups = Vec::new();
             for (config, deqp) in configs.iter().zip(deqp.iter()) {
-                let mut groups = config.test_groups(deqp)?;
+                let mut groups = config.test_groups(deqp, &suite.include)?;
                 test_groups.append(&mut groups);
             }
 
