@@ -16,7 +16,7 @@ use rand::rngs::StdRng;
 use rand::seq::SliceRandom;
 use rand::SeedableRng;
 use rayon::prelude::*;
-use regex::RegexSet;
+use regex::{Regex, RegexSet};
 use std::collections::{HashMap, HashSet};
 use std::fs::File;
 use std::io::prelude::*;
@@ -180,7 +180,7 @@ pub trait Deqp {
 
                     for test in remaining_tests {
                         caselist_results.push(RunnerResult {
-                            test: test.name().to_owned(),
+                            test: format!("{}{}", self.prefix(), &test.name()),
                             status: RunnerStatus::Missing,
                             duration: Default::default(),
                             subtest: false,
@@ -750,6 +750,77 @@ impl Deqp for PiglitCommand {
     }
 }
 
+impl DeqpCommand {
+    pub fn qpa_text_check(&self, testcase: &str, regex: &str, log_name: &str) -> Result<bool> {
+        let regex = Regex::new(regex)
+            .with_context(|| format!("Compiling QPA text check RE '{}'", regex))?;
+        let qpa_path = self.output_dir.join(format!("{}.qpa", testcase));
+
+        let mut args: Vec<String> = Vec::new();
+
+        // Add on the user's specified deqp arguments.
+        for arg in &self.args {
+            args.push(arg.clone());
+        }
+
+        args.push(format!("--deqp-case={}", &testcase));
+        add_filename_arg(&mut args, "--deqp-log-filename", &qpa_path)
+            .context("adding log to args")?;
+
+        let output = Command::new(&self.deqp)
+            .current_dir(self.deqp.parent().unwrap_or_else(|| Path::new("/")))
+            .stdout(Stdio::piped())
+            .stderr(Stdio::piped())
+            .stdin(Stdio::null())
+            .args(args)
+            // Disable MESA_DEBUG output by default for debug Mesa builds, which
+            // otherwise fills the logs with warnings about GL errors that are
+            // thrown (you're running deqp!  Of course it makes GL errors!)
+            .env("MESA_DEBUG", "silent")
+            .envs(self.env.iter())
+            .output()
+            .with_context(|| format!("Failed to spawn {}", &self.deqp.display()))?;
+
+        if !output.status.success() {
+            anyhow::bail!(
+                "Failed to invoke dEQP for renderer check:\nstdout:\n{}\nstderr:\n{}",
+                String::from_utf8_lossy(&output.stdout),
+                String::from_utf8_lossy(&output.stderr)
+            );
+        }
+
+        let mut qpa = String::new();
+        File::open(qpa_path)
+            .context("opening renderer check QPA")?
+            .read_to_string(&mut qpa)
+            .context("reading renderer check QPA")?;
+
+        let renderer = if testcase == "dEQP-VK.info.device" {
+            qpa.lines()
+                .find(|x| x.contains("deviceName: "))
+                .context("finding deviceName in QPA output")?
+                .trim_start_matches("deviceName: ")
+        } else {
+            // XML regexing :(
+            let text_re = Regex::new("<Text>(.*)</Text>").context("text match RE")?;
+            let text_line = qpa
+                .lines()
+                .skip_while(|line| !line.contains("<TestCaseResult"))
+                .nth(1)
+                .context("reading line after TestCaseResult")?;
+
+            let captures = text_re
+                .captures(&text_line)
+                .context("Finding text line after TestCaseResult")?;
+            captures.get(1).unwrap().as_str()
+        };
+
+        println!("{}: {}", log_name, &renderer);
+
+        Ok(regex.is_match(renderer))
+    }
+}
+
 fn results_collection<W: Write>(
     status_output: &mut W,
     run_results: &mut RunnerResults,
@@ -862,13 +933,26 @@ where
     let mut lines: Vec<String> = Vec::new();
 
     for path in files {
+        let path = path.as_ref();
         for line in BufReader::new(
-            File::open(&path)
-                .with_context(|| format!("opening path: {}", path.as_ref().display()))?,
+            File::open(&path).with_context(|| format!("opening path: {}", path.display()))?,
         )
         .lines()
         {
-            lines.push(line.context("reading line")?);
+            let line = line.with_context(|| format!("reading line from {}", path.display()))?;
+            // In newer dEQP, vk-master.txt just contains a list of .txt
+            // caselist files relative to its current path, so recursively read
+            // thoseand append their contents.
+            if line.ends_with(".txt") {
+                let sub_path = path.parent().context("Getting path parent dir")?.join(line);
+
+                lines.extend_from_slice(
+                    &read_lines(&[sub_path.as_path()])
+                        .with_context(|| format!("reading sub-caselist {}", sub_path.display()))?,
+                );
+            } else {
+                lines.push(line)
+            }
         }
     }
     Ok(lines)
