@@ -15,6 +15,13 @@ struct RunnerCommandResult {
     results: Result<RunnerResults>,
 }
 
+pub fn tempfile<S: AsRef<str>>(data: S) -> Result<tempfile::TempPath> {
+    let mut file = tempfile::NamedTempFile::new().context("creating tempfile")?;
+    file.write(data.as_ref().as_bytes())
+        .context("writing tempfile")?;
+    Ok(file.into_temp_path())
+}
+
 pub fn lines_tempfile<S: AsRef<str>, I: IntoIterator<Item = S>>(
     lines: I,
 ) -> Result<tempfile::TempPath> {
